@@ -19,7 +19,7 @@
 
   Packet processing module
 
-  $Id: process.c,v 1.27 2003/08/26 13:30:07 bengen Exp $
+  $Id: process.c,v 1.28 2004/04/06 08:49:35 bengen Exp $
 */
 
 #include <sys/time.h>
@@ -86,14 +86,28 @@ uid_t get_tcp_local_uid(u_int32_t, u_int32_t, u_int16_t, u_int16_t);
 
 struct localuiddata * read_proc_file(char* filename);
 
-unsigned int HASH(unsigned int s, unsigned int d, unsigned short int sp, unsigned short int dp, unsigned char pt)
+/* A modified jhash_3words from include/linux/jhash.h */
+u_int32_t HASH(u_int32_t a, u_int32_t b, u_int16_t sp, u_int16_t dp, u_int8_t pt)
 {
-  return ( ( ( (ENABLED(OUT_SRC)?(unsigned long long)ntohl(s):0LL) |
-	       (ENABLED(OUT_DST)?(unsigned long long)ntohl(d)<<32:0LL) ) ^
-	     ( (ENABLED(OUT_SRCPORT)?(unsigned long long)sp<<16:0LL) |
-	       (ENABLED(OUT_DSTPORT)?(unsigned long long)dp<<48:0LL) |
-	       (ENABLED(OUT_PROTO)?(unsigned long long)pt:0LL) ) )
- 	   % cfg->hash_table_size );
+  /* The golden ration: an arbitrary value */
+#define JHASH_GOLDEN_RATIO	0x9e3779b9
+
+  u_int32_t c;
+  a += JHASH_GOLDEN_RATIO;
+  b += JHASH_GOLDEN_RATIO;
+  c = (sp << 16) + dp + pt + cfg->hash_initval;
+
+  a -= b; a -= c; a ^= (c>>13);
+  b -= c; b -= a; b ^= (a<<8);
+  c -= a; c -= b; c ^= (b>>13);
+  a -= b; a -= c; a ^= (c>>12);
+  b -= c; b -= a; b ^= (a<<16);
+  c -= a; c -= b; c ^= (b>>5);
+  a -= b; a -= c; a ^= (c>>3);
+  b -= c; b -= a; b ^= (a<<10);
+  c -= a; c -= b; c ^= (b>>15);
+
+  return ( c & cfg->hash_mask);
 }
 
 void reopen_socket(void)
